function S = fun(X0,NLP,y0,n,nh,M,fx)
%Funo auxiliar para a utilizao do MINLP com restries de igualdade
%
%  usage: [F] = FUN(X0,NLP,Y0,N,nh,M)
%
%  F  : valor da funo objetivo
%  X0 : vetor das variveis (length(X0)==N para problema no relaxado e
%       length(X0)==N+M para o problema relaxado.
%  NLP: arquivo como descrito em MINLP
%  Y0 : vetor das variveis binrias
%  N  : nmero de variveis contnuas no problema
%  nh : nmero de restries de igualdade
%  M  : penalidade das restries

% adaptado de fun.m

if nargin < 7
   fx=[];
end

fx = sort(fx);

if n==length(X0) %no relaxado
   [F, G] = eval([NLP '(X0,y0)']);
   S = F + M*max([abs(G(1:nh)) G(nh+1:end) 0]);
   return;
else %problema relaxado em algumas ou todas as variveis binrias
   index = 1:length(y0);
   index(fx-n) = []; %removendo os fixos
   y = y0;
   y(index) = X0(n+1:end);
end

x = X0(1:n); % x  sempre as n primeiras variveis

[F, G] = eval([NLP '(x,y)']); %problema relaxado
S = F + M*max([abs(G(1:nh)) G(nh+1:end) 0]);

